/******************************************************************************
 * Copyright (C) 2018 Xilinx, Inc.  All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * Use of the Software is limited solely to applications:
 * (a) running on a Xilinx device, or
 * (b) that interact with a Xilinx device through a bus or interconnect.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * XILINX  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * Except as contained in this notice, the name of the Xilinx shall not be used
 * in advertising or otherwise to promote the sale, use or other dealings in
 * this Software without prior written authorization from Xilinx.
 ******************************************************************************/

/***************************** Include Files ********************************/
#include "xpfw_default.h"
#include "xpfw_config.h"
#include "xpfw_core.h"
#include "xpfw_module.h"

#include "xpfw_mod_plsysmon.h"

/************************** Constant Definitions ****************************/

/*
 * The following constants map to the XPAR parameters created in the
 * xparameters.h file. They are defined here such that a user can easily
 * change all the needed parameters in one place.
 */
#define SYSMON_DEVICE_ID 	XPAR_XSYSMONPSU_0_DEVICE_ID

/*
 * Time in ms for checking PL Voltage and Temperature values
 */
#define PL_VOLT_AND_TEMP_CHECK_TIME	10000U

const XPfw_Module_t *PlSysmonModPtr;

static XSysMonPsu SysMonInst;      /* System Monitor driver instance */

XSysMonPsu_Config *ConfigPtr;
XSysMonPsu *SysMonInstPtr = &SysMonInst;

/****************************************************************************/
/**
 *
 * This function converts the fraction part of the given floating point number
 * (after the decimal point)to an integer.
 *
 * @param	FloatNum is the floating point number.
 *
 * @return	Integer number to a precision of 3 digits.
 *
 * @note
 * This function is used in the printing of floating point data to a STDIO device
 * using the xil_printf function. The xil_printf is a very small foot-print
 * printf function and does not support the printing of floating point numbers.
 *
 *****************************************************************************/
int SysMonPsuFractionToInt(float FloatNum)
{
	float Temp;

	Temp = FloatNum;
	if (FloatNum < 0) {
		Temp = -(FloatNum);
	}

	return( ((int)((Temp -(float)((int)Temp)) * (1000.0f))));
}

/****************************************************************************/
/**
*
* This function reads the latest on-chip temperature and various voltages
* using System Monitor driver APIs.
*
* @param	None
*
* @return	None
*
* @note   	None
*
****************************************************************************/
static void XPfw_PlSysmonRead(void)
{
	u32 TempRawData;
	u32 VccAuxRawData;
	u32 VccIntRawData;
	float TempData;
	float VccAuxData;
	float VccIntData;
	float MaxData;
	float MinData;

	/* Wait till the End of Sequence occurs */
	while ((XSysMonPsu_IntrGetStatus(SysMonInstPtr) & ((u64)XSYSMONPSU_ISR_1_EOS_MASK<< 32)) !=
			((u64)XSYSMONPSU_ISR_1_EOS_MASK<< 32));

	/*
	 * Read the on-chip Temperature Data (Current/Maximum/Minimum)
	 * from the ADC data registers.
	 */
	TempRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, XSM_CH_TEMP, XSYSMON_PL);
	TempData = XSysMonPsu_RawToTemperature_OnChip(TempRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current PL Temperature is %0d.%03d Centigrades.\r\n",
			(int)(TempData), SysMonPsuFractionToInt(TempData));


	TempRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr, XSM_MAX_TEMP, XSYSMON_PL);
	MaxData = XSysMonPsu_RawToTemperature_OnChip(TempRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Maximum PL Temperature is %0d.%03d Centigrades. \r\n",
			(int)(MaxData), SysMonPsuFractionToInt(MaxData));

	TempRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr, XSM_MIN_TEMP, XSYSMON_PL);
	MinData = XSysMonPsu_RawToTemperature_OnChip(TempRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Minimum PL Temperature is %0d.%03d Centigrades. \r\n",
			(int)(MinData), SysMonPsuFractionToInt(MinData));

	/*
	 * Read the VP_VN Votage Data (Current/Maximum/Minimum) from the
	 * ADC data registers.
	 */
	VccIntRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, XSM_CH_VPVN, XSYSMON_PL);
	VccIntData = XSysMonPsu_RawToVoltage(VccIntRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current VP_VN is %0d.%03d Volts. \r\n",
			(int)(VccIntData), SysMonPsuFractionToInt(VccIntData));

	/*
	 * Read the VREFP Votage Data (Current/Maximum/Minimum) from the
	 * ADC data registers.
	 */
	VccIntRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, XSM_CH_VREFP, XSYSMON_PL);
	VccIntData = XSysMonPsu_RawToVoltage(VccIntRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current VREFP is %0d.%03d Volts. \r\n",
			(int)(VccIntData), SysMonPsuFractionToInt(VccIntData));

	/*
	 * Read the VREFN Votage Data (Current/Maximum/Minimum) from the
	 * ADC data registers.
	 */
	VccIntRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, XSM_CH_VREFN, XSYSMON_PL);
	VccIntData = XSysMonPsu_RawToVoltage(VccIntRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current VREFN is %0d.%03d Volts. \r\n",
			(int)(VccIntData), SysMonPsuFractionToInt(VccIntData));

	/*
	 * Read the PL VCCINT Votage Data (Current/Maximum/Minimum) from the
	 * ADC data registers.
	 */
	VccIntRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, XSM_CH_SUPPLY1, XSYSMON_PL);
	VccIntData = XSysMonPsu_RawToVoltage(VccIntRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current PL VCCINT is %0d.%03d Volts. \r\n",
			(int)(VccIntData), SysMonPsuFractionToInt(VccIntData));

	VccIntRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr,
			XSM_MAX_SUPPLY1, XSYSMON_PL);
	MaxData = XSysMonPsu_RawToVoltage(VccIntRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Maximum PL VCCINT is %0d.%03d Volts. \r\n",
			(int)(MaxData), SysMonPsuFractionToInt(MaxData));

	VccIntRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr,
			XSM_MIN_SUPPLY1, XSYSMON_PL);
	MinData = XSysMonPsu_RawToVoltage(VccIntRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Minimum PL VCCINT is %0d.%03d Volts. \r\n",
			(int)(MinData), SysMonPsuFractionToInt(MinData));

	/*
	 * Read the PL VCCAUX Votage Data (Current/Maximum/Minimum) from the
	 * ADC data registers.
	 */
	VccIntRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, XSM_CH_SUPPLY2, XSYSMON_PL);
	VccIntData = XSysMonPsu_RawToVoltage(VccIntRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current PL VCCAUX is %0d.%03d Volts. \r\n",
			(int)(VccIntData), SysMonPsuFractionToInt(VccIntData));

	VccIntRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr,
			XSM_MAX_SUPPLY2, XSYSMON_PL);
	MaxData = XSysMonPsu_RawToVoltage(VccIntRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Maximum PL VCCAUX is %0d.%03d Volts. \r\n",
			(int)(MaxData), SysMonPsuFractionToInt(MaxData));

	VccIntRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr,
			XSM_MIN_SUPPLY2, XSYSMON_PL);
	MinData = XSysMonPsu_RawToVoltage(VccIntRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Minimum PL VCCAUX is %0d.%03d Volts. \r\n",
			(int)(MinData), SysMonPsuFractionToInt(MinData));

	/*
	 * Read the PL VCCBRAM Votage Data (Current/Maximum/Minimum) from the
	 * ADC data registers.
	 */
	VccAuxRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, XSM_CH_SUPPLY3, XSYSMON_PL);
	VccAuxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current PL VCCBRAM is %0d.%03d Volts. \r\n",
			(int)(VccAuxData), SysMonPsuFractionToInt(VccAuxData));

	VccAuxRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr,
			XSM_MAX_SUPPLY3, XSYSMON_PL);
	MaxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Maximum PL VCCBRAM is %0d.%03d Volts. \r\n",
			(int)(MaxData), SysMonPsuFractionToInt(MaxData));


	VccAuxRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr,
			XSM_MIN_SUPPLY3, XSYSMON_PL);
	MinData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Minimum PL VCCBRAM is %0d.%03d Volts. \r\n\r\n",
			(int)(MinData), SysMonPsuFractionToInt(MinData));

	/*
	 * Read the VCC_PSINTLP Votage Data (Current/Maximum/Minimum) from the
	 * ADC data registers.
	 */
	VccAuxRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, XSM_CH_SUPPLY4, XSYSMON_PL);
	VccAuxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current VCC_PSINTLP is %0d.%03d Volts. \r\n",
			(int)(VccAuxData), SysMonPsuFractionToInt(VccAuxData));

	VccAuxRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr,
			XSM_MAX_SUPPLY4, XSYSMON_PL);
	MaxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Maximum VCC_PSINTLP is %0d.%03d Volts. \r\n",
			(int)(MaxData), SysMonPsuFractionToInt(MaxData));


	VccAuxRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr,
			XSM_MIN_SUPPLY4, XSYSMON_PL);
	MinData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Minimum VCC_PSINTLP is %0d.%03d Volts. \r\n\r\n",
			(int)(MinData), SysMonPsuFractionToInt(MinData));

	/*
	 * Read the VCC_PSINTFP Votage Data (Current/Maximum/Minimum) from the
	 * ADC data registers.
	 */
	VccAuxRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, XSM_CH_SUPPLY5, XSYSMON_PL);
	VccAuxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current VCC_PSINTFP is %0d.%03d Volts. \r\n",
			(int)(VccAuxData), SysMonPsuFractionToInt(VccAuxData));

	VccAuxRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr,
			XSM_MAX_SUPPLY5, XSYSMON_PL);
	MaxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Maximum VCC_PSINTFP is %0d.%03d Volts. \r\n",
			(int)(MaxData), SysMonPsuFractionToInt(MaxData));


	VccAuxRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr,
			XSM_MIN_SUPPLY5, XSYSMON_PL);
	MinData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Minimum VCC_PSINTFP is %0d.%03d Volts. \r\n\r\n",
			(int)(MinData), SysMonPsuFractionToInt(MinData));

	/*
	 * Read the VCC_PSAUX Votage Data (Current/Maximum/Minimum) from the
	 * ADC data registers.
	 */
	VccAuxRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, XSM_CH_SUPPLY6, XSYSMON_PL);
	VccAuxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current VCC_PSAUX is %0d.%03d Volts. \r\n",
			(int)(VccAuxData), SysMonPsuFractionToInt(VccAuxData));

	VccAuxRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr,
			XSM_MAX_SUPPLY6, XSYSMON_PL);
	MaxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Maximum VCC_PSAUX is %0d.%03d Volts. \r\n",
			(int)(MaxData), SysMonPsuFractionToInt(MaxData));


	VccAuxRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr,
			XSM_MIN_SUPPLY6, XSYSMON_PL);
	MinData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Minimum VCC_PSAUX is %0d.%03d Volts. \r\n\r\n",
			(int)(MinData), SysMonPsuFractionToInt(MinData));

	/*
	 * Read the VAUX0 Votage Data from the
	 * ADC data registers.
	 */
	VccAuxRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, XSM_CH_AUX_MIN, XSYSMON_PL);
	VccAuxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current VAUX0 is %0d.%03d Volts. \r\n",
			(int)(VccAuxData), SysMonPsuFractionToInt(VccAuxData));

	/*
	 * Read the VAUX1 Votage Data from the
	 * ADC data registers.
	 */
	VccAuxRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, 17U, XSYSMON_PL);
	VccAuxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current VAUX1 is %0d.%03d Volts. \r\n",
			(int)(VccAuxData), SysMonPsuFractionToInt(VccAuxData));

	/*
	 * Read the VAUX2 Votage Data from the
	 * ADC data registers.
	 */
	VccAuxRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, 18U, XSYSMON_PL);
	VccAuxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current VAUX2 is %0d.%03d Volts. \r\n",
			(int)(VccAuxData), SysMonPsuFractionToInt(VccAuxData));

	/*
	 * Read the VAUX3 Votage Data from the
	 * ADC data registers.
	 */
	VccAuxRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, 19U, XSYSMON_PL);
	VccAuxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current VAUX3 is %0d.%03d Volts. \r\n",
			(int)(VccAuxData), SysMonPsuFractionToInt(VccAuxData));

	/*
	 * Read the VAUX4 Votage Data from the
	 * ADC data registers.
	 */
	VccAuxRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, 20U, XSYSMON_PL);
	VccAuxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current VAUX4 is %0d.%03d Volts. \r\n",
			(int)(VccAuxData), SysMonPsuFractionToInt(VccAuxData));

	/*
	 * Read the VAUX5 Votage Data from the
	 * ADC data registers.
	 */
	VccAuxRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, 21U, XSYSMON_PL);
	VccAuxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current VAUX5 is %0d.%03d Volts. \r\n",
			(int)(VccAuxData), SysMonPsuFractionToInt(VccAuxData));

	/*
	 * Read the VAUX6 Votage Data from the
	 * ADC data registers.
	 */
	VccAuxRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, 22U, XSYSMON_PL);
	VccAuxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current VAUX6 is %0d.%03d Volts. \r\n",
			(int)(VccAuxData), SysMonPsuFractionToInt(VccAuxData));

	/*
	 * Read the VAUX7 Votage Data from the
	 * ADC data registers.
	 */
	VccAuxRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, 23U, XSYSMON_PL);
	VccAuxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current VAUX7 is %0d.%03d Volts. \r\n",
			(int)(VccAuxData), SysMonPsuFractionToInt(VccAuxData));

	/*
	 * Read the VAUX8 Votage Data from the
	 * ADC data registers.
	 */
	VccAuxRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, 24U, XSYSMON_PL);
	VccAuxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current VAUX8 is %0d.%03d Volts. \r\n",
			(int)(VccAuxData), SysMonPsuFractionToInt(VccAuxData));

	/*
	 * Read the VAUX9 Votage Data (Current/Maximum/Minimum) from the
	 * ADC data registers.
	 */
	VccAuxRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, 25U, XSYSMON_PL);
	VccAuxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current VAUX9 is %0d.%03d Volts. \r\n",
			(int)(VccAuxData), SysMonPsuFractionToInt(VccAuxData));

	/*
	 * Read the VAUXA Votage Data from the
	 * ADC data registers.
	 */
	VccAuxRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, 26U, XSYSMON_PL);
	VccAuxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current VAUXA is %0d.%03d Volts. \r\n",
			(int)(VccAuxData), SysMonPsuFractionToInt(VccAuxData));

	/*
	 * Read the VAUXB Votage Data from the
	 * ADC data registers.
	 */
	VccAuxRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, 27U, XSYSMON_PL);
	VccAuxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current VAUXB is %0d.%03d Volts. \r\n",
			(int)(VccAuxData), SysMonPsuFractionToInt(VccAuxData));

	/*
	 * Read the VAUXC Votage Data from the
	 * ADC data registers.
	 */
	VccAuxRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, 28U, XSYSMON_PL);
	VccAuxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current VAUXC is %0d.%03d Volts. \r\n",
			(int)(VccAuxData), SysMonPsuFractionToInt(VccAuxData));

	/*
	 * Read the VAUXD Votage Data from the
	 * ADC data registers.
	 */
	VccAuxRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, 29U, XSYSMON_PL);
	VccAuxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current VAUXD is %0d.%03d Volts. \r\n",
			(int)(VccAuxData), SysMonPsuFractionToInt(VccAuxData));

	/*
	 * Read the VAUXE Votage Data from the
	 * ADC data registers.
	 */
	VccAuxRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, 30U, XSYSMON_PL);
	VccAuxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current VAUXE is %0d.%03d Volts. \r\n",
			(int)(VccAuxData), SysMonPsuFractionToInt(VccAuxData));

	/*
	 * Read the VAUXF Votage Data from the
	 * ADC data registers.
	 */
	VccAuxRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, XSM_CH_AUX_MAX, XSYSMON_PL);
	VccAuxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current VAUXF is %0d.%03d Volts. \r\n",
			(int)(VccAuxData), SysMonPsuFractionToInt(VccAuxData));

	/*
	 * Read the VUser0 Votage Data (Current/Maximum/Minimum) from the
	 * ADC data registers.
	 */
	VccAuxRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, XSM_CH_SUPPLY7, XSYSMON_PL);
	VccAuxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current VUser0 is %0d.%03d Volts. \r\n",
			(int)(VccAuxData), SysMonPsuFractionToInt(VccAuxData));

	VccAuxRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr,
			XSM_MAX_SUPPLY7, XSYSMON_PL);
	MaxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Maximum VUser0 is %0d.%03d Volts. \r\n",
			(int)(MaxData), SysMonPsuFractionToInt(MaxData));


	VccAuxRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr,
			XSM_MIN_SUPPLY7, XSYSMON_PL);
	MinData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Minimum VUser0 is %0d.%03d Volts. \r\n\r\n",
			(int)(MinData), SysMonPsuFractionToInt(MinData));

	/*
	 * Read the VUser1 Votage Data (Current/Maximum/Minimum) from the
	 * ADC data registers.
	 */
	VccAuxRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, XSM_CH_SUPPLY8, XSYSMON_PL);
	VccAuxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current VUser1 is %0d.%03d Volts. \r\n",
			(int)(VccAuxData), SysMonPsuFractionToInt(VccAuxData));

	VccAuxRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr,
			XSM_MAX_SUPPLY8, XSYSMON_PL);
	MaxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Maximum VUser1 is %0d.%03d Volts. \r\n",
			(int)(MaxData), SysMonPsuFractionToInt(MaxData));


	VccAuxRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr,
			XSM_MIN_SUPPLY8, XSYSMON_PL);
	MinData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Minimum VUser1 is %0d.%03d Volts. \r\n\r\n",
			(int)(MinData), SysMonPsuFractionToInt(MinData));

	/*
	 * Read the VUser2 Votage Data (Current/Maximum/Minimum) from the
	 * ADC data registers.
	 */
	VccAuxRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, XSM_CH_SUPPLY9, XSYSMON_PL);
	VccAuxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current VUser2 is %0d.%03d Volts. \r\n",
			(int)(VccAuxData), SysMonPsuFractionToInt(VccAuxData));

	VccAuxRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr,
			XSM_MAX_SUPPLY9, XSYSMON_PL);
	MaxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Maximum VUser2 is %0d.%03d Volts. \r\n",
			(int)(MaxData), SysMonPsuFractionToInt(MaxData));


	VccAuxRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr,
			XSM_MIN_SUPPLY9, XSYSMON_PL);
	MinData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Minimum VUser2 is %0d.%03d Volts. \r\n\r\n",
			(int)(MinData), SysMonPsuFractionToInt(MinData));

	/*
	 * Read the VUser3 Votage Data (Current/Maximum/Minimum) from the
	 * ADC data registers.
	 */
	VccAuxRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, XSM_CH_SUPPLY10, XSYSMON_PL);
	VccAuxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current VUser3 is %0d.%03d Volts. \r\n",
			(int)(VccAuxData), SysMonPsuFractionToInt(VccAuxData));

	VccAuxRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr,
			XSM_MAX_SUPPLY10, XSYSMON_PL);
	MaxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Maximum VUser3 is %0d.%03d Volts. \r\n",
			(int)(MaxData), SysMonPsuFractionToInt(MaxData));


	VccAuxRawData = XSysMonPsu_GetMinMaxMeasurement(SysMonInstPtr,
			XSM_MIN_SUPPLY10, XSYSMON_PL);
	MinData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "The Minimum VUser3 is %0d.%03d Volts. \r\n\r\n",
			(int)(MinData), SysMonPsuFractionToInt(MinData));

	/*
	 * Read the PL VCCADC (VCCAMS) Votage Data (Current/Maximum/Minimum) from the
	 * ADC data registers.
	 */
	VccAuxRawData = XSysMonPsu_GetAdcData(SysMonInstPtr, XSM_CH_VCCAMS, XSYSMON_PL);
	VccAuxData = XSysMonPsu_RawToVoltage(VccAuxRawData);
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nThe Current PL VCCADC is %0d.%03d Volts. \r\n",
			(int)(VccAuxData), SysMonPsuFractionToInt(VccAuxData));
}

/****************************************************************************/
/**
*
* This function is called when PlSysmon Module is initialized
*
* This function does the following tasks:
*	- Initiate the System Monitor device driver instance
*	- Run self-test on the device
*	- Setup the sequence registers to continuously monitor on-chip
*	temperature, various voltages
*	- Setup configuration registers to start the sequence
*
* @note   	None
*
****************************************************************************/
static void PlSysmonCfgInit(const XPfw_Module_t *ModPtr, const u32 *CfgData, u32 Len)
{
	s32 Status = XST_FAILURE;
	u64 Channels;

	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nPL Sysmon Module Init \r\n");

	/* Initialize the SysMon driver. */
	ConfigPtr = XSysMonPsu_LookupConfig(SYSMON_DEVICE_ID);
	if (ConfigPtr == NULL) {
		XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nSysmon driver init failed. ");
		goto END;
	}
	XSysMonPsu_CfgInitialize(SysMonInstPtr, ConfigPtr,
			ConfigPtr->BaseAddress);

	/* Self Test the System Monitor device */
	Status = XSysMonPsu_SelfTest(SysMonInstPtr);
	if (Status != XST_SUCCESS) {
		XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nSysmon Selftest failed. ");
		goto END;
	}
	XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nSysmon Selftest passed. ");

	/*
	 * Disable the Channel Sequencer before configuring the Sequence
	 * registers.
	 */
	XSysMonPsu_SetSequencerMode(SysMonInstPtr, XSM_SEQ_MODE_SAFE, XSYSMON_PL);


	/* Disable all the alarms in the Configuration Register 1. */
	XSysMonPsu_SetAlarmEnables(SysMonInstPtr, 0x0, XSYSMON_PL);


	/*
	 * Setup the Averaging to be done for the channels in the
	 * Configuration 0 register as 16 samples:
	 */
	XSysMonPsu_SetAvg(SysMonInstPtr, XSM_AVG_16_SAMPLES, XSYSMON_PL);

	/*
	 * Setup the Sequence register for 1st Auxiliary channel
	 * Setting is:
	 *	- Add acquisition time by 6 ADCCLK cycles.
	 *	- Bipolar Mode
	 *
	 * Setup the Sequence register for 16th Auxiliary channel
	 * Setting is:
	 *	- Add acquisition time by 6 ADCCLK cycles.
	 *	- Unipolar Mode
	 */
	Status = XSysMonPsu_SetSeqInputMode(SysMonInstPtr, XSYSMONPSU_SEQ_CH0_VP_VN_MASK |
			((XSYSMONPSU_SEQ_CH1_VAUX00_MASK | XSYSMONPSU_SEQ_CH1_VAUX01_MASK |
			XSYSMONPSU_SEQ_CH1_VAUX02_MASK | XSYSMONPSU_SEQ_CH1_VAUX03_MASK |
			XSYSMONPSU_SEQ_CH1_VAUX04_MASK | XSYSMONPSU_SEQ_CH1_VAUX05_MASK |
			XSYSMONPSU_SEQ_CH1_VAUX06_MASK | XSYSMONPSU_SEQ_CH1_VAUX07_MASK |
			XSYSMONPSU_SEQ_CH1_VAUX08_MASK | XSYSMONPSU_SEQ_CH1_VAUX09_MASK |
			XSYSMONPSU_SEQ_CH1_VAUX0A_MASK | XSYSMONPSU_SEQ_CH1_VAUX0B_MASK |
			XSYSMONPSU_SEQ_CH1_VAUX0C_MASK | XSYSMONPSU_SEQ_CH1_VAUX0D_MASK |
			XSYSMONPSU_SEQ_CH1_VAUX0E_MASK | XSYSMONPSU_SEQ_CH1_VAUX0F_MASK )
			<< 16), XSYSMON_PL);
	if (Status != XST_SUCCESS) {
		XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nSysMonPsu_SetSeqInputMode failed.");
		goto END;
	}

	Status = XSysMonPsu_SetSeqAcqTime(SysMonInstPtr,XSYSMONPSU_SEQ_CH0_VP_VN_MASK |
			((XSYSMONPSU_SEQ_CH1_VAUX00_MASK | XSYSMONPSU_SEQ_CH1_VAUX01_MASK |
			XSYSMONPSU_SEQ_CH1_VAUX02_MASK | XSYSMONPSU_SEQ_CH1_VAUX03_MASK |
			XSYSMONPSU_SEQ_CH1_VAUX04_MASK | XSYSMONPSU_SEQ_CH1_VAUX05_MASK |
			XSYSMONPSU_SEQ_CH1_VAUX06_MASK | XSYSMONPSU_SEQ_CH1_VAUX07_MASK |
			XSYSMONPSU_SEQ_CH1_VAUX08_MASK | XSYSMONPSU_SEQ_CH1_VAUX09_MASK |
			XSYSMONPSU_SEQ_CH1_VAUX0A_MASK | XSYSMONPSU_SEQ_CH1_VAUX0B_MASK |
			XSYSMONPSU_SEQ_CH1_VAUX0C_MASK | XSYSMONPSU_SEQ_CH1_VAUX0D_MASK |
			XSYSMONPSU_SEQ_CH1_VAUX0E_MASK | XSYSMONPSU_SEQ_CH1_VAUX0F_MASK )
			<< 16), XSYSMON_PL);
	if (Status != XST_SUCCESS) {
		XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nSysMonPsu_SetSeqAcqTime failed.");
		goto END;
	}


	/*
	 * Enable the averaging on the following channels in the Sequencer
	 * registers:
	 * 	- On-chip Temperature, VCCINT/VCCAUX  supply sensors
	 * 	- 1st/16th Auxiliary Channels
	 *	- Calibration Channel
	 */
	Channels = (u64)(XSYSMONPSU_SEQ_CH0_SUP1_MASK |
			XSYSMONPSU_SEQ_CH0_SUP2_MASK |
			XSYSMONPSU_SEQ_CH0_SUP3_MASK |
			XSYSMONPSU_SEQ_CH0_SUP4_MASK |
			XSYSMONPSU_SEQ_CH0_SUP5_MASK |
			XSYSMONPSU_SEQ_CH0_SUP6_MASK |
			XSYSMONPSU_SEQ_CH0_VP_VN_MASK |
			XSYSMONPSU_SEQ_CH0_VREFP_MASK |
			XSYSMONPSU_SEQ_CH0_VREFN_MASK |
			((u64)(XSYSMONPSU_SEQ_CH1_VAUX00_MASK |
			XSYSMONPSU_SEQ_CH1_VAUX01_MASK |
			XSYSMONPSU_SEQ_CH1_VAUX02_MASK |
			XSYSMONPSU_SEQ_CH1_VAUX03_MASK |
			XSYSMONPSU_SEQ_CH1_VAUX04_MASK |
			XSYSMONPSU_SEQ_CH1_VAUX05_MASK |
			XSYSMONPSU_SEQ_CH1_VAUX06_MASK |
			XSYSMONPSU_SEQ_CH1_VAUX07_MASK |
			XSYSMONPSU_SEQ_CH1_VAUX08_MASK |
			XSYSMONPSU_SEQ_CH1_VAUX09_MASK |
			XSYSMONPSU_SEQ_CH1_VAUX0A_MASK |
			XSYSMONPSU_SEQ_CH1_VAUX0B_MASK |
			XSYSMONPSU_SEQ_CH1_VAUX0C_MASK |
			XSYSMONPSU_SEQ_CH1_VAUX0D_MASK |
			XSYSMONPSU_SEQ_CH1_VAUX0E_MASK |
			XSYSMONPSU_SEQ_CH1_VAUX0F_MASK) << 16) |
			((u64)(XSYSMONPSU_SEQ_CH2_SUP7_MASK |
			XSYSMONPSU_SEQ_CH2_SUP8_MASK |
			XSYSMONPSU_SEQ_CH2_SUP9_MASK |
			XSYSMONPSU_SEQ_CH2_SUP10_MASK |
			XSYSMONPSU_SEQ_CH2_VCCAMS_MASK) << (u64)32));

	Status =  XSysMonPsu_SetSeqAvgEnables(SysMonInstPtr, XSYSMONPSU_SEQ_CH0_TEMP_MASK |
			Channels |
			XSYSMONPSU_SEQ_CH0_CALIBRTN_MASK, XSYSMON_PL);
	if (Status != XST_SUCCESS) {
		XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nSysMonPsu_SetSeqAvgEnables failed.");
		goto END;
	}

	/*
	 * Enable the following channels in the Sequencer registers:
	 * 	- On-chip Temperature, VCCINT/VCCAUX supply sensors
	 * 	- 1st/16th Auxiliary Channel
	 *	- Calibration Channel
	 */
	Status =  XSysMonPsu_SetSeqChEnables(SysMonInstPtr, XSYSMONPSU_SEQ_CH0_TEMP_MASK |
			Channels |
			XSYSMONPSU_SEQ_CH0_CALIBRTN_MASK, XSYSMON_PL);
	if (Status != XST_SUCCESS) {
		XPfw_Printf(DEBUG_PRINT_ALWAYS, "\r\nSysMonPsu_SetSeqChEnables failed.");
		goto END;
	}


	/*
	 * Set the ADCCLK frequency equal to 1/32 of System clock for the System
	 * Monitor in the Configuration Register 2.
	 */
	XSysMonPsu_SetAdcClkDivisor(SysMonInstPtr, 32, XSYSMON_PL);

	/* Enable the Channel Sequencer in continuous sequencer cycling mode. */
	XSysMonPsu_SetSequencerMode(SysMonInstPtr, XSM_SEQ_MODE_CONTINPASS, XSYSMON_PL);

	Status = XPfw_CoreScheduleTask(ModPtr, PL_VOLT_AND_TEMP_CHECK_TIME, XPfw_PlSysmonRead);
	if (XST_FAILURE == Status) {
		XPfw_Printf(DEBUG_PRINT_ALWAYS,"\r\nPLSYSMON (MOD-%d):Scheduling "
				"Sysmon read task failed.", ModPtr->ModId);
	}
END:
	if (Status != XST_SUCCESS) {
		XPfw_Printf(DEBUG_PRINT_ALWAYS, "Failed to Initialize PL Sysmon Module\r\n");
	}
}


void ModPlSysmonInit(void)
{
	PlSysmonModPtr = XPfw_CoreCreateMod();

	(void)XPfw_CoreSetCfgHandler(PlSysmonModPtr, PlSysmonCfgInit);
}
